program Servo;

{
Program to control a servo connected to a parralel port.
You should connect the signal line of the servo to D0 (Pin 2) on the
parralel port. GND D0 (Pin 18) should be connected to GND. I use a
transistor to do the switching (For safety, the specs say you can only source
2.4 mA from the parralel port). The connections to the parralel connector are
shown for a DB25 connector.

Here's the schematic

   +5V >-------------+--------> To plus on servo
                     |
                    ---
                    | |
                    | | R=1500 Ohms
                    | |
                    ---
                     |
                     +--------> To signal line of servo.
        R=5600 Ohms  |C
                     /
          |-----|  |/ NPN-type transistror (Cheap switch transistor)
D0     >--|     |--+  A common type used in Europe is the BC547.
(Pin 2)   |-----| B|\
                     \E
                     |
D0 GND >--------+----+--------> To GND of servo.
(Pin 18)        |
              --+-- GND
               ---
                -

!!  WARNING  !!  WARNING  !!  WARNING  !!  WARNING  !!  WARNING  !!  WARNING  !!

Don't use your parralel port for providing the power. It'll blow your IO-Card
because the motor's in your servo can easy draw several hundreds milliamperes.
Use a 5 Volt power supply instead. (6 V won't hurt either)

!!  WARNING  !!  WARNING  !!  WARNING  !!  WARNING  !!  WARNING  !!  WARNING  !!

brand       pulse timing          servo leads

            min   neut  max   Hz  Plus   Gnd     Signal

graupner    0.8   1.5   2.2   50  red    brown   orange
multiplex   1.05  1.6   2.15  40  red    black   yellow
futuba      0.9   1.5   2.1   50  red    black   white
robbe       0.65  1.3   1.95  50  red    black   white
simprop     1.2   1.7   2.2   50  red    blue    black

Replace the following constants with your own values
RepFreq - The frequecy at which the pulse is repeated
          It's the Hz column from the table above
Min     - Minimum pulse duration. Divide 1 by the value found in the table.
          ie Multiplex                                  1
          Min=1.05                                  ---------
          Then the constant in the program should be        -3
                                                     1.05*10
          And round it to the nearest integer.
          In this case that's 465.
Max     - Maximum pulse duration. Furter the same as above.
Neut    - Neutral pulse duration. Furter the same as above.

Keys that work are
Arrow-Up    Very small step one direction
Arrow-Down  Very small step other direction
Arrow-Left  Extreme postition one side
Arrow-Right Extreme postition other side
Pg-Up       Step one direction
Pg-Dn       Step other direction
Home        Neutral position
Esc         End

Disclaimer
There's no WARRANTY. This program comes 'as is'.
So if it mucks up your servo, blast your parralel port away or it destroy's
your house, I'm not responsible.

Date        Rev  Changes
26/05/96    0.1  Creation

For questions, tips, money, ferrari's, pentium's, etc.
Just e-mail me at soepb500@hio.tem.nhl.nl

PS  My excuses for the misspelled words. I'm not a native english.
PS2 Sorry for all the procedures, but I copied them from some units I made.
}
uses
  Crt, Dos;

const
  { Printer port, default at $378 }
  PPort = $378;

  { Some handy ones }
  On    = true;
  Off   = false;

  { Keyboeard constants }
  kbDown      = $5000;
  kbEsc       = $011B;
  kbHome      = $4700;
  kbLeft      = $4B00;
  kbPgDn      = $5100;
  kbPgUp      = $4900;
  kbRight     = $4D00;
  kbUp        = $4800;

  { Described above }
  RepFreq     = 40;
  Max         = 950;
  Min         = 465;
  Neut        = 625;

type
  TProc=procedure;

var
  Timer: boolean;

  OldProc,TimerProc:TProc;
  Oldticks,Ticks:word;

  Puls, Trig, Einde: boolean;
  Key, NewFreq: word;

{ Used if no intterrupt procedure is set, ie InitTimer(F,nil) }
procedure Dummy;far;
begin
end;

{ Change the timer frequency to Freq }
procedure ChangeFreq(Freq:word);
var
  Ticks:word;
begin
  if Freq>0 then
  begin
    Ticks:=round(1193180/freq);
    Oldticks:=round($FFFF/ticks);
  end
  else Ticks:=0;

  asm
    cli
    mov    al,00110110b
    out    43h,al
    jmp    @1
@1: mov    ax,ticks
    out    40h,al
    jmp    @2
@2: xchg   ah,al
    out    40h,al
    jmp    @3
@3: sti
  end;
end;

{ This get's called every time a puls should be generated }
procedure Int; far; assembler;
asm
{
this procedure is the same as in assembler, just a little more
comprehensible.

  if Pulse then
  begin
    if Trig then
    begin
      Port[PPort]:=1;
      Trig:=False;
      Puls:=False;
      ChangeFreq(RepFreq);
    end
    else
    begin
      Trig:=True;
      Port[PPort]:=0;
    end;
  end
  else
  begin
    Puls:=True;
    ChangeFreq(NewFreq);
  end;
}
    cmp  Puls,True
    jne  @2
    cmp  Trig,True
    jne  @1
    mov  dx,PPort
    mov  al,1
    out  dx,al
    mov  Trig,False
    mov  Puls,False
    push RepFreq
    call ChangeFreq
    jmp  @3
@1: mov  Trig,True
    mov  dx,PPort
    mov  al,0
    out  dx,al
    jmp  @3
@2: mov  Puls,True
    push NewFreq
    call ChangeFreq
@3:
end;

{ This is the procedure that get's the actually timer interrupt }
procedure TimerInt; interrupt; assembler;
asm
{
  if Timer then TimerProc;
  Dec(Ticks)
  if Ticks=0 then
  begin
    Ticks:=OldTicks;
    asm pushf end;
    OldProc;
  end;
  Port[$20]:=$20;
}
@nc:    cmp	Timer,On
	jne	@off
	call	TimerProc

@off:	dec	Ticks
	jnz	@NoOld

	mov	ax,OldTicks
	mov	Ticks,ax

        pushf
	call	OldProc

@NoOld: mov	al,20h
	out	20h,al
end;

{ Init's the timer. Set's the interrupt vectors etc }
procedure InitTimer(Freq:word;Proc:TProc);
begin
  Timer:=Off;

  if Addr(Proc)=nil then
    TimerProc:=Dummy
  else
    TimerProc:=Proc;

  GetIntVec($8,@OldProc);	{ Init }
  SetIntVec($8,@TimerInt);
  ChangeFreq(Freq);
  Timer:=On;
end;

{ Reset's the timer system to it's original dosstate }
procedure ResetTimer;
begin
  Timer:=Off;
  ChangeFreq(0);
  SetIntVec($8,@OldProc); { Exit }
end;

{ Get's scancode }
function GetKey: word; assembler;
asm
    mov	  ah,0
    int	  16h
end;

{
MIN     NEUTRAL MAX
1,05 ms-1,6 ms- 2,15 ms ;Puls duration
950     625     465     ;Freq of timer to generate puls
}
begin
  ClrScr;
  Trig:=False;
  Puls:=False;
  NewFreq:=Neut; { Neutral }

  InitTimer(RepFreq, Int);

  repeat
    GotoXY(1,1);
    WriteLn(NewFreq:4);
    if KeyPressed then
    begin
      Key:=GetKey;
      case Key of
        kbEsc   : Einde:=True;
        kbUp    : if NewFreq<Max then Inc(NewFreq,1);
        kbDown  : if NewFreq>Min then Dec(NewFreq,1);
        kbPgUp  : if NewFreq<Max then Inc(NewFreq,10);
        kbPgDn  : if NewFreq>Min then Dec(NewFreq,10);
        kbLeft  : NewFreq:=Max;
        kbRight : NewFreq:=Min;
        kbHome  : NewFreq:=Neut;
      end;
    end;
  until Einde;

  ResetTimer;
end.